﻿using LogicModule.Nodes.Helpers;
using LogicModule.ObjectModel;
using LogicModule.ObjectModel.TypeSystem;

namespace LogicNodesSDK.Logic.Examples
{
  /// <summary>
  /// A code example for a very basic logic node.
  /// Forwards the input signal to the output.
  /// </summary>
  public class BasicNode : LogicNodeBase
  {
    /// <summary>
    /// Initializes a new instance of the <see cref="BasicNode"/> class.
    /// </summary>
    /// <param name="context">The node context.</param>
    public BasicNode(INodeContext context)
    : base(context)
    {
      context.ThrowIfNull("context");

      // Get the TypeService from the context
      var typeService = context.GetService<ITypeService>();

      // Initialize the ports Input and Output with a double value object and the translatable name "Input" / "Output".
      // The port type 'Number' accepts connections to any other number.
      this.Input = typeService.CreateDouble(PortTypes.Number, "Input");
      this.Output = typeService.CreateDouble(PortTypes.Number, "Output");
    }

    /// <summary>
    /// A double value object as input.
    /// </summary>
    [Input]
    public DoubleValueObject Input { get; private set; }

    /// <summary>
    /// A double value object as output.
    /// </summary>
    [Output]
    public DoubleValueObject Output { get; private set; }

    /// <summary>
    /// This function is called every time any input (marked by attribute [Input]) receives a value and no input has no value.
    /// The inputs that were updated for this function to be called, have <see cref="IValueObject.WasSet"/> set to true. After this function returns 
    /// the <see cref="IValueObject.WasSet"/> flag will be reset to false.
    /// </summary>
    public override void Execute()
    {
      // Setting the output will also set the WasSet flag of the output
      this.Output.Value = this.Input.Value;
    }

    /// <summary>
    /// This function is called only once when the logic page is being loaded.
    /// The base function of this is empty. 
    /// </summary>
    public override void Startup()
    {
      base.Startup();
    }

    /// <summary>
    /// By default this function gets the translation for the node's in- and output from the <see cref="LogicNodeBase.ResourceManager"/>.
    /// A resource file with translation is required for this to work.
    /// </summary>
    /// <param name="language">The requested language, for example "en" or "de".</param>
    /// <param name="key">The key to translate.</param>
    /// <returns>The translation of <paramref name="key"/> in the requested language, or <paramref name="key"/> if the translation is missing.</returns>
    public override string Localize(string language, string key)
    {
      return base.Localize(language, key);
    }
  }
}
