﻿using System.Collections.Generic;
using LogicModule.Nodes.Helpers;
using LogicModule.ObjectModel;
using LogicModule.ObjectModel.TypeSystem;

namespace LogicNodesSDK.Logic.Examples
{
  /// <summary>
  /// A code example using an EnumValueObject.
  /// Converts a 3byte RGB value (DPT 232.600) from or to 3x 1byte color values.
  /// </summary>
  public class ColorConverter : LogicNodeBase
  {
    /// <summary>
    /// The possible directions to convert the colors.
    /// </summary>
    private static class Directions
    {
      public const string OneToThree = "DirectionOneToThree";
      public const string ThreeToOne = "DirectionThreeToOne";
    }

    /// <summary>
    /// Available colors to use as input / output.
    /// </summary>
    private static class Colors
    {
      public const string RGB = "RGB";
      public const string Red = "Red";
      public const string Green = "Green";
      public const string Blue = "Blue";
    }

    /// <summary>
    /// The type service.
    /// </summary>
    private readonly ITypeService typeService;

    /// <summary>
    /// Initializes a new instance of the <see cref="ColorConverter"/> class.
    /// </summary>
    /// <param name="context">The node context.</param>
    public ColorConverter(INodeContext context)
    : base(context)
    {
      context.ThrowIfNull("context");

      this.typeService = context.GetService<ITypeService>();

      // initialize inputs and outputs as an empty list of undefined value objects
      this.Inputs = new List<IValueObject>();
      this.Outputs = new List<IValueObject>();

      // create an array of possible enum values
      string[] directions = new[]
      {
        Directions.OneToThree,
        Directions.ThreeToOne
      };

      this.Direction = this.typeService.CreateEnum("Direction", "Direction", directions);
      // set function to call when value is changed for Direction
      this.Direction.ValueSet += this.DirectionOnValueSet;

      // set value, so the inputs and outputs are set in the OnValueSet function
      this.Direction.Value = Directions.OneToThree;
    }

    /// <summary>
    /// The direction to convert the colors.
    /// The attribute <see cref="ParameterAttribute.AsTitle"/> is set to true, so the direction can be set by clicking the icon.
    /// </summary>
    [Parameter(AsTitle = true, IsDefaultShown = false)]
    public EnumValueObject Direction { get; private set; }

    /// <summary>
    /// The list of inputs.
    /// Either 3x 1 byte or 1x 3 byte.
    /// </summary>
    [Input]
    public IList<IValueObject> Inputs { get; private set; }

    /// <summary>
    /// The list of outputs.
    /// Either 3x 1 byte or 1x 3 byte.
    /// </summary>
    [Output]
    public IList<IValueObject> Outputs { get; private set; }

    /// <summary>
    /// Calculate the output(s) based on the input(s).
    /// </summary>
    public override void Execute()
    {
      if (this.Direction == Directions.OneToThree)
      {
        uint rgb = this.Inputs[0].Value as uint? ?? 0;
        this.Outputs[0].Value = (rgb >> 16) & 0xff;
        this.Outputs[1].Value = (rgb >> 8) & 0xff;
        this.Outputs[2].Value = rgb & 0xff;
      }
      else
      {
        byte red = this.Inputs[0].Value as byte? ?? 0;
        byte green = this.Inputs[1].Value as byte? ?? 0;
        byte blue = this.Inputs[2].Value as byte? ?? 0;
        this.Outputs[0].Value = (red << 16) + (green << 8) + blue;
      }
    }

    /// <summary>
    /// Configure the input(s) and output(s) based on the chosen direction to convert the colors.
    /// </summary>
    /// <param name="sender">The sender object.</param>
    /// <param name="args">The event arguments.</param>
    private void DirectionOnValueSet(object sender, ValueChangedEventArgs args)
    {
      // clear the list of inputs and outputs first
      this.Inputs.Clear();
      this.Outputs.Clear();

      // instead of reading the field Direction here, it is also possible to read the new value from the event arguments
      bool oneToThree = this.Direction.Value == Directions.OneToThree;

      // list1 is the side with 1x 3 byte
      var list1 = oneToThree ? this.Inputs : this.Outputs;
      list1.Add(this.typeService.CreateUInt(PortTypes.DWord, Colors.RGB, 0));

      // list2 is the side with 3x 1 byte
      var list2 = oneToThree ? this.Outputs : this.Inputs;
      foreach (var color in new string[] { Colors.Red, Colors.Green, Colors.Blue })
      {
        list2.Add(this.typeService.CreateByte(PortTypes.Byte, color, 0));
      }

      // limit the RGB input value to 3 byte (0xFFFFFF)
      if (oneToThree)
      {
        ((UIntValueObject)this.Inputs[0]).MaxValue = 0xffffff;
      }
    }
  }
}
